#include <iostream>
#include <conio.h>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"

using namespace std;
struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data() {
    Input in;
    size_t number_count;
    cin >> number_count;
    in.numbers.resize(number_count);
    for (size_t i = 0; i < number_count; i++) {
        cin >> in.numbers[i];
    }
    cin >> in.bin_count;
    return in;
}

void find_minmax(const vector<double>& numbers, double& min, double& max) {
    min = numbers[0];
    max = numbers[0];
    for (double x : numbers)
    {
        if (x < min)
        {
            min = x;
        }
        if (x > max)
        {
            max = x;
        }
    }
}

vector<size_t> make_histogram(vector<double>& numbers, size_t bin_count)
{
    double min_number, max_number;
    find_minmax(numbers, min_number, max_number);
    vector<size_t> bins(bin_count);
    for (int i = 0; i < bin_count; i++)
    {
        bins[i] = 0;
    }
    double bin_size = (max_number - min_number) / bin_count;
    for (size_t i = 0; i < numbers.size(); i++)
    {
        bool found = false;
        for (size_t j = 0; (j < bin_count - 1) && !found; j++)
        {
            auto lo = min_number + j * bin_size;
            auto hi = min_number + (j + 1) * bin_size;
            if ((lo <= numbers[i]) && (numbers[i] < hi))
            {
                bins[j]++;
                found = true;
            }
        }
        if (!found)
        {
            bins[bin_count - 1]++;
        }
    }
    return bins;
}

void find_max_capacity(vector<size_t>& bins, size_t& max_bin_capacity)
{
    max_bin_capacity = bins[0];
    for (size_t x : bins)
    {
        if (x > max_bin_capacity)
        {
            max_bin_capacity = x;
        }
    }
}

void show_histogram_text(vector<size_t>& bins, size_t max_asterisk)
{
    size_t max_bin_capacity{};
    find_max_capacity(bins, max_bin_capacity);
    size_t height = 0;
    for (int i = 0; i < bins.size(); i++)
    {
        if (bins[i] < 10)
        {
            cout << "  " << bins[i] << "|";
        }
        if ((bins[i] < 100) && (bins[i] > 9))
        {
            cout << " " << bins[i] << "|";
        }
        if (bins[i] >= 100)
        {
            cout << bins[i] << "|";
        }
        if (max_bin_capacity > max_asterisk)
        {
            height = max_asterisk * (static_cast<double>(bins[i]) / max_bin_capacity);
        }
        else
        {
            height = bins[i];
        }
        for (int j = 0; j < height; j++)
        {
            cout << "*";
        }
        cout << "\n";
    }
}

int main()
{
    const size_t SCREEN_WIDTH = 80;
    const size_t MAX_ASTERISK = SCREEN_WIDTH - 3 - 1;
    auto in = input_data();
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
    return 0;
}
